if (not Grid) then return end

-- Libraries

local L = AceLibrary("AceLocale-2.2"):new("GridLayoutPlus")
local RL = AceLibrary("Roster-2.1")

-- New Grid module

GridLayoutPlus = Grid:NewModule("GridLayoutPlus", "AceConsole-2.0")

GridLayoutPlus.defaultDB = {
	autoLayout = true,
	withPets = true,
	withTanks = true,
	maxColumns = 16,
	warnLayoutChanged = true,
	warnPlayersHidden = true,
	warnPetsHidden = true,
	warnTanksHidden = true,
}


-- Init

local ORDER_PLUS = 40

function GridLayoutPlus:OnEnable()
	local options = GridLayout.options.args
	options["PlusHeader"] = {
		type = "header",
		order = ORDER_PLUS,
	}
	ORDER_PLUS = ORDER_PLUS + 1
	options["autoLayout"] = {
		type = "toggle",
		name = L["Auto adjust layout (+)"],
		desc = L["Automatically adjust layout to fit raid size"],
		order = ORDER_PLUS,
		get = function ()
			return GridLayout.db.profile.autoLayout
		end,
		set = function (v)
			GridLayout.db.profile.autoLayout = v
			GridLayoutPlus:UpdateLayout()
		end,
	}
	ORDER_PLUS = ORDER_PLUS + 1
	options["withPets"] = {
		type = "toggle",
		name = L["Use pet layouts (+)"],
		desc = L["Use pet layouts when auto adjusting (if possible)"],
		order = ORDER_PLUS,
		get = function ()
			return GridLayout.db.profile.withPets
		end,
		set = function (v)
			GridLayout.db.profile.withPets = v
			GridLayoutPlus:UpdateLayout()
		end,
	}
	ORDER_PLUS = ORDER_PLUS + 1
	options["withTanks"] = {
		type = "toggle",
		name = L["Use tank layouts (+)"],
		desc = L["Use tank layouts when auto adjusting (if possible)"],
		order = ORDER_PLUS,
		get = function ()
			return GridLayout.db.profile.withTanks
		end,
		set = function (v)
			GridLayout.db.profile.withTanks = v
			GridLayoutPlus:UpdateLayout()
		end,
	}
	ORDER_PLUS = ORDER_PLUS + 1
	options["maxColumns"] = {
		type = "range",
		name = L["Max number of columns (+)"],
		desc = L["Maximum number of columns when auto-adjusting (WARNING: May hide units)"],
		order = ORDER_PLUS,
		min = 1,
		max = 16,
		step = 1,
		get = function ()
			return GridLayout.db.profile.maxColumns
		end,
		set = function (v)
			GridLayout.db.profile.maxColumns = v
			GridLayoutPlus:UpdateLayout()
		end,
	}
	ORDER_PLUS = ORDER_PLUS + 1
	options["warnLayoutChanged"] = {
		type = "toggle",
		name = L["Warn when layout changes (+)"],
		desc = L["Show warning when the layout is automatically changed"],
		order = ORDER_PLUS,
		get = function ()
			return GridLayout.db.profile.warnLayoutChanged
		end,
		set = function (v)
			GridLayout.db.profile.warnLayoutChanged = v
			GridLayoutPlus:UpdateLayout()
		end,
	}
	ORDER_PLUS = ORDER_PLUS + 1
	options["warnPlayersHidden"] = {
		type = "toggle",
		name = L["Warn if players are hidden (+)"],
		desc = L["Show warning when player units are forced to be hidden"],
		order = ORDER_PLUS,
		get = function ()
			return GridLayout.db.profile.warnPlayersHidden
		end,
		set = function (v)
			GridLayout.db.profile.warnPlayersHidden = v
			GridLayoutPlus:UpdateLayout()
		end,
	}
	ORDER_PLUS = ORDER_PLUS + 1
	options["warnPetsHidden"] = {
		type = "toggle",
		name = L["Warn if pets are hidden (+)"],
		desc = L["Show warning when pet units are forced to be hidden"],
		order = ORDER_PLUS,
		get = function ()
			return GridLayout.db.profile.warnPetsHidden
		end,
		set = function (v)
			GridLayout.db.profile.warnPetsHidden = v
			GridLayoutPlus:UpdateLayout()
		end,
	}
	ORDER_PLUS = ORDER_PLUS + 1
	options["warnTanksHidden"] = {
		type = "toggle",
		name = L["Warn if tanks are hidden (+)"],
		desc = L["Show warning when tank units are forced to be hidden"],
		order = ORDER_PLUS,
		get = function ()
			return GridLayout.db.profile.warnTanksHidden
		end,
		set = function (v)
			GridLayout.db.profile.warnTanksHidden = v
			GridLayoutPlus:UpdateLayout()
		end,
	}
	self:RegisterEvent("RosterLib_RosterUpdated", "UpdateLayout")
	if oRA then
		self:RegisterEvent("oRA_MainTankUpdate", "UpdateLayout")
		self:RegisterEvent("oRA_JoinedRaid", "UpdateLayout")
	end
	if CT_RATarget then
		hooksecurefunc("CT_RAOptions_UpdateMTs", function() self:UpdateLayout() end)
	end
end


-- Additional Layouts

GridLayout:AddLayout(L["By Group 40 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		groupFilter = "2",
	},
	[3] = {
		groupFilter = "3",
	},
	[4] = {
		groupFilter = "4",
	},
	[5] = {
		groupFilter = "5",
	},
	[6] = {
		groupFilter = "6",
	},
	[7] = {
		groupFilter = "7",
	},
	[8] = {
		groupFilter = "8",
	},
	[9] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 35 (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		groupFilter = "2",
	},
	[3] = {
		groupFilter = "3",
	},
	[4] = {
		groupFilter = "4",
	},
	[5] = {
		groupFilter = "5",
	},
	[6] = {
		groupFilter = "6",
	},
	[7] = {
		groupFilter = "7",
	},
})

GridLayout:AddLayout(L["By Group 35 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		groupFilter = "2",
	},
	[3] = {
		groupFilter = "3",
	},
	[4] = {
		groupFilter = "4",
	},
	[5] = {
		groupFilter = "5",
	},
	[6] = {
		groupFilter = "6",
	},
	[7] = {
		groupFilter = "7",
	},
	[8] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 30 (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		groupFilter = "2",
	},
	[3] = {
		groupFilter = "3",
	},
	[4] = {
		groupFilter = "4",
	},
	[5] = {
		groupFilter = "5",
	},
	[6] = {
		groupFilter = "6",
	},
})

GridLayout:AddLayout(L["By Group 30 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		groupFilter = "2",
	},
	[3] = {
		groupFilter = "3",
	},
	[4] = {
		groupFilter = "4",
	},
	[5] = {
		groupFilter = "5",
	},
	[6] = {
		groupFilter = "6",
	},
	[7] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 20 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		groupFilter = "2",
	},
	[3] = {
		groupFilter = "3",
	},
	[4] = {
		groupFilter = "4",
	},
	[5] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 15 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		groupFilter = "2",
	},
	[3] = {
		groupFilter = "3",
	},
	[4] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 5 (+)"], {
	[1] = {
		groupFilter = "1",
	},
})

GridLayout:AddLayout(L["By Group 5 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
	},
	[2] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Sorted Class (+)"], {
	[1] = {
		groupFilter = "WARRIOR,ROGUE,DRUID,PALADIN,SHAMAN,PRIEST,MAGE,WARLOCK,HUNTER",
		groupBy = "CLASS",
		groupingOrder = "WARRIOR,ROGUE,DRUID,PALADIN,SHAMAN,PRIEST,MAGE,WARLOCK,HUNTER",
		unitsPerColumn = 5,
		maxColumns = 8,
	},
})

GridLayout:AddLayout(L["By Sorted Class w/Pets (+)"], {
	[1] = {
		groupFilter = "WARRIOR,ROGUE,DRUID,PALADIN,SHAMAN,PRIEST,MAGE,WARLOCK,HUNTER",
		groupBy = "CLASS",
		groupingOrder = "WARRIOR,ROGUE,DRUID,PALADIN,SHAMAN,PRIEST,MAGE,WARLOCK,HUNTER",
		unitsPerColumn = 5,
		maxColumns = 8,
	},
	[2] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		groupBy = "CLASS",
		groupingOrder = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	}
})

GridLayout:AddLayout(L["By Group 25 w/BlizzTanks (+)"], {
	[1] = {
		groupFilter = "MAINTANK,MAINASSIST",
		groupingOrder = "MAINTANK,MAINASSIST",
	},
	[2] = {
		groupFilter = "1",
	},
	[3] = {
		groupFilter = "2",
	},
	[4] = {
		groupFilter = "3",
	},
	[5] = {
		groupFilter = "4",
	},
	[6] = {
		groupFilter = "5",
	},
})

GridLayout:AddLayout(L["By Group 25 w/BlizzTanks w/Pets (+)"], {
	[1] = {
		groupFilter = "MAINTANK,MAINASSIST",
		groupingOrder = "MAINTANK,MAINASSIST",
	},
	[2] = {
		groupFilter = "1",
	},
	[3] = {
		groupFilter = "2",
	},
	[4] = {
		groupFilter = "3",
	},
	[5] = {
		groupFilter = "4",
	},
	[6] = {
		groupFilter = "5",
	},
	[7] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 10 w/BlizzTanks (+)"], {
	[1] = {
		groupFilter = "MAINTANK,MAINASSIST",
		groupingOrder = "MAINTANK,MAINASSIST",
	},
	[2] = {
		groupFilter = "1",
	},
	[3] = {
		groupFilter = "2",
	},
})

GridLayout:AddLayout(L["By Group 10 w/BlizzTanks w/Pets (+)"], {
	[1] = {
		groupFilter = "MAINTANK,MAINASSIST",
		groupingOrder = "MAINTANK,MAINASSIST",
	},
	[2] = {
		groupFilter = "1",
	},
	[3] = {
		groupFilter = "2",
	},
	[4] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayoutPlus.RATanks25 = {
	[1] = {
		nameList = "",
		sortMethod = "INDEX",
	},
	[2] = {
		groupFilter = "1",
	},
	[3] = {
		groupFilter = "2",
	},
	[4] = {
		groupFilter = "3",
	},
	[5] = {
		groupFilter = "4",
	},
	[6] = {
		groupFilter = "5",
	},
}

GridLayoutPlus.RATanks25Pets = {
	[1] = {
		nameList = "",
		sortMethod = "INDEX",
	},
	[2] = {
		groupFilter = "1",
	},
	[3] = {
		groupFilter = "2",
	},
	[4] = {
		groupFilter = "3",
	},
	[5] = {
		groupFilter = "4",
	},
	[6] = {
		groupFilter = "5",
	},
	[7] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
}

GridLayoutPlus.RATanks10 = {
	[1] = {
		nameList = "",
		sortMethod = "INDEX",
	},
	[2] = {
		groupFilter = "1",
	},
	[3] = {
		groupFilter = "2",
	},
}

GridLayoutPlus.RATanks10Pets = {
	[1] = {
		nameList = "",
		sortMethod = "INDEX",
	},
	[2] = {
		groupFilter = "1",
	},
	[3] = {
		groupFilter = "2",
	},
	[4] = {
		isPetGroup = true,
		groupFilter = "HUNTER,WARLOCK",
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
}

GridLayout:AddLayout(L["By Group 25 w/RATanks (+)"], GridLayoutPlus.RATanks25)
GridLayout:AddLayout(L["By Group 25 w/RATanks w/Pets (+)"], GridLayoutPlus.RATanks25Pets)
GridLayout:AddLayout(L["By Group 10 w/RATanks (+)"], GridLayoutPlus.RATanks10)
GridLayout:AddLayout(L["By Group 10 w/RATanks w/Pets (+)"], GridLayoutPlus.RATanks10Pets)

-- Event handlers

function GridLayoutPlus:UpdateLayout()
	if GridLayout.db.profile.autoLayout and (GetNumRaidMembers() > 0) then
		local maxsubgroup = 0
		local newlayout = nil
		local newmaxraidmembers = 40
		local newwithpets = true
		local newwithtanks = true
		local numberofpotentialpets = 0
		local tanklist = nil
		for u in RL:IterateRoster(nil) do
			if u.subgroup > maxsubgroup then
				maxsubgroup = u.subgroup
			end
			if (u.class == "HUNTER") or (u.class == "WARLOCK") then
				numberofpotentialpets = numberofpotentialpets + 1
			end
		end
		if oRA and oRA.maintanktable then
			tanklist = table.concat(oRA.maintanktable, ',')
		elseif CT_RATarget and CT_RATarget.MainTanks then
			tanklist = table.concat(CT_RATarget.MainTanks, ',')
		end
		if tanklist then
			GridLayoutPlus.RATanks25[1].nameList = tanklist    
			GridLayoutPlus.RATanks25Pets[1].nameList = tanklist
			GridLayoutPlus.RATanks10[1].nameList = tanklist    
			GridLayoutPlus.RATanks10Pets[1].nameList = tanklist        
		end
		if string.find(GridLayout.db.profile.layout, L["By Group"]) then
			if (maxsubgroup > 7) and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > (maxsubgroup + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 40 w/Pets (+)"]
				newwithtanks = nil
			elseif (maxsubgroup > 7) and (GridLayout.db.profile.maxColumns > 7) then
				newlayout = L["By Group 40"]
				newwithpets = nil
				newwithtanks = nil
			elseif (maxsubgroup > 6) and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > (maxsubgroup + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 35 w/Pets (+)"]
				newmaxraidmembers = 35
				newwithtanks = nil
			elseif (maxsubgroup > 6) and (GridLayout.db.profile.maxColumns > 6) then
				newlayout = L["By Group 35 (+)"]
				newmaxraidmembers = 35
				newwithpets = nil
				newwithtanks = nil
			elseif (maxsubgroup > 5) and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > (maxsubgroup + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 30 w/Pets (+)"]
				newmaxraidmembers = 30
				newwithtanks = nil
			elseif (maxsubgroup > 5) and (GridLayout.db.profile.maxColumns > 5) then
				newlayout = L["By Group 30 (+)"]
				newmaxraidmembers = 30
				newwithpets = nil
				newwithtanks = nil
			elseif (maxsubgroup > 2) and GridLayout.db.profile.withTanks and tanklist and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > (maxsubgroup+1 + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 25 w/RATanks w/Pets (+)"]
				newmaxraidmembers = 25
			elseif (maxsubgroup > 2) and GridLayout.db.profile.withTanks and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > (maxsubgroup+1 + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 25 w/BlizzTanks w/Pets (+)"]
				newmaxraidmembers = 25
			elseif (maxsubgroup > 4) and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > 5) then
				newlayout = L["By Group 25 w/Pets"]
				newmaxraidmembers = 25
				newwithtanks = nil
			elseif (maxsubgroup > 2) and GridLayout.db.profile.withTanks and tanklist and (GridLayout.db.profile.maxColumns > 5) then
				newlayout = L["By Group 25 w/RATanks (+)"]
				newmaxraidmembers = 25
			elseif (maxsubgroup > 2) and GridLayout.db.profile.withTanks and (GridLayout.db.profile.maxColumns > (maxsubgroup+1 + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 25 w/BlizzTanks (+)"]
				newmaxraidmembers = 25
			elseif (maxsubgroup > 4) and (GridLayout.db.profile.maxColumns > 4) then
				newlayout = L["By Group 25"]
				newmaxraidmembers = 25
				newwithpets = nil
				newwithtanks = nil
			elseif (maxsubgroup > 3) and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > (maxsubgroup + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 20 w/Pets (+)"]
				newmaxraidmembers = 20
				newwithtanks = nil
			elseif (maxsubgroup > 3) and (GridLayout.db.profile.maxColumns > 3) then
				newlayout = L["By Group 20"]
				newmaxraidmembers = 20
				newwithpets = nil
				newwithtanks = nil
			elseif (maxsubgroup > 2) and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > (maxsubgroup + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 15 w/Pets (+)"]
				newmaxraidmembers = 15
				newwithtanks = nil
			elseif (maxsubgroup > 2) and (GridLayout.db.profile.maxColumns > 2) then
				newlayout = L["By Group 15"]
				newmaxraidmembers = 15
				newwithpets = nil
				newwithtanks = nil
			elseif GridLayout.db.profile.withTanks and tanklist and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > (maxsubgroup+1 + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 10 w/RATanks w/Pets (+)"]
				newmaxraidmembers = 10
			elseif GridLayout.db.profile.withTanks and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > (maxsubgroup+1 + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 10 w/BlizzTanks w/Pets (+)"]
				newmaxraidmembers = 10
			elseif (maxsubgroup > 1) and GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > (maxsubgroup + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 10 w/Pets"]
				newmaxraidmembers = 10
				newwithtanks = nil
			elseif GridLayout.db.profile.withTanks and tanklist and (GridLayout.db.profile.maxColumns > 2) then
				newlayout = L["By Group 10 w/RATanks (+)"]
				newmaxraidmembers = 10
			elseif GridLayout.db.profile.withTanks and (GridLayout.db.profile.maxColumns > 2) then
				newlayout = L["By Group 10 w/BlizzTanks (+)"]
				newmaxraidmembers = 10
			elseif (maxsubgroup > 1) and (GridLayout.db.profile.maxColumns > 1) then
				newlayout = L["By Group 10"]
				newmaxraidmembers = 10
				newwithpets = nil
				newwithtanks = nil
			elseif GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > (maxsubgroup + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 5 w/Pets (+)"]
				newmaxraidmembers = 5
				newwithtanks = nil
			else
				newlayout = L["By Group 5 (+)"]
				newmaxraidmembers = 5
				newwithpets = nil
				newwithtanks = nil
			end
		elseif string.find(GridLayout.db.profile.layout, L["By Sorted Class"]) then
			if GridLayout.db.profile.withPets and (GridLayout.db.profile.maxColumns > ((GetNumRaidMembers() / 5) + (numberofpotentialpets / 5))) then
				newlayout = L["By Sorted Class w/Pets (+)"]
			else
				newlayout = L["By Sorted Class (+)"]
				newwithpets = nil
			end
		end
		if newlayout and (GridLayout.db.profile.layout ~= newlayout) then
			GridLayout.db.profile.layout = newlayout
			GridLayout:LoadLayout(newlayout)
			if GridLayout.db.profile.warnLayoutChanged then
				self:Print(L["Your layout has automatically changed"])
			end
			if GridLayout.db.profile.warnPlayersHidden and (GetNumRaidMembers() > newmaxraidmembers) then
				self:Print(L["Your settings are causing player units to be hidden"])
			end
			if GridLayout.db.profile.warnPetsHidden and GridLayout.db.profile.withPets and (not newwithpets) and (numberofpotentialpets > 0) then
				self:Print(L["Your settings are causing pet units to be hidden"])
			end
			if GridLayout.db.profile.warnTanksHidden and GridLayout.db.profile.withTanks and tanklist and (not newwithtanks) then
				self:Print(L["Your settings are causing tank units to be hidden"])
			end
		end
	end
end
